from datetime import datetime as dt

from . import client as MeetupClient
from ..certificator import BaseCertificator
from .models import Event


class MeetupCertificator(BaseCertificator):
    def __init__(self, urlname, event_id, api_key, **kwargs):
        super().__init__(**kwargs)
        self.urlname = urlname
        self.event_id = event_id
        self.client = MeetupClient(api_key=api_key)

    def get_certificate_data(self):
        attendances = self.client.get_attendances(self.urlname, self.event_id)
        return ({'name': attendance['member']['name']} for attendance in attendances)

    def get_meta(self):
        event_data = self.client.get_event(self.urlname, self.event_id)
        event = Event(**event_data)
        event.clean()
        return {
            'city': event.venue['city'],
            'date': dt.strftime(event.date, '%d/%m/%Y'),
            'full_date': event.full_date,
            'organizer': event.group['name'],
            'place': event.venue['name'],
            'title': event.name,
            'workload': event.duration,
        }



import os.path

from prettyconf import config

PATH = os.path.abspath(os.path.dirname(__file__))
TEMPLATES_PATH = os.path.join(PATH, 'templates')
MEETUP_API_KEY = config('MEETUP_API_KEY', default='')



import requests


class MeetupClient:
    def __init__(self, api_key):
        self.params = {'key': api_key}

    def get_attendances(self, urlname, event_id):
        url = 'https://api.meetup.com/{}/events/{}/attendance'.format(urlname, event_id)
        response = requests.get(url, params=self.params)
        return response.json()

    def get_event(self, urlname, event_id):
        url = 'https://api.meetup.com/{}/events/{}'.format(urlname, event_id)
        response = requests.get(url, self.params)
        return response.json()

    def get_member(self, urlname, member_id):
        url = 'https://api.meetup.com/{}/members/{}'.format(urlname, member_id)
        response = requests.get(url, self.params)
        return response.json()



from datetime import datetime as dt
from simple_model import Model


class Event(Model):
    fields = (
        'venue',
        'group',
        'duration',
        'time',
        'name',
    )

    MONTHS = (
        'janeiro',
        'fevereiro',
        'março',
        'abril',
        'maio',
        'junho',
        'julho',
        'agosto',
        'setembro',
        'outubro',
        'novembro',
        'dezembro',
    )

    def clean_duration(self, value):
        if not value:
            return 'não especificado'

        return value / (60 * 60 * 1000)

    def clean_time(self, value):
        return value / 1000

    @property
    def date(self):
        return dt.fromtimestamp(self.time)

    @property
    def full_date(self):
        full_date = dt.strftime(self.date, '%d de {} de %Y')
        full_month = self.MONTHS[self.date.month - 1].title()
        return full_date.format(full_month)



import csv
import json
import os.path

from jinja2 import Environment, FileSystemLoader, select_autoescape
from weasyprint import HTML

from . import config


class BaseCertificator:
    def __init__(self, destination_path='.', template_path=None, template_filename='template.html',
                 filename_format='certificate-{id:0>3}.pdf'):
        self.template_path = template_path
        self.destination_path = destination_path
        self.template_filename = template_filename
        self.filename_format = filename_format

    def get_meta(self):
        raise NotImplementedError

    def get_certificate_data(self):
        raise NotImplementedError

    @property
    def template_path(self):
        return self._template_path

    @template_path.setter
    def template_path(self, path):
        if not path:
            self._template_path = path
            return

        assert os.path.exists(path), 'You must provide an existing folder with the correct permissions'

        path = os.path.expanduser(path)
        self._template_path = path

    def get_template_paths(self):
        paths = [
            os.path.abspath('.'),
            os.path.abspath('./templates'),
            config.TEMPLATES_PATH,
        ]

        if not self.template_path:
            return paths

        return [self.template_path] + paths

    @property
    def template(self):
        paths = self.get_template_paths()
        env = Environment(
            loader=FileSystemLoader(paths),
            autoescape=select_autoescape(['html', 'xml']),
        )
        return env.get_template(self.template_filename)

    def get_context(self, **kwargs):
        context = {}
        meta = self.get_meta()

        context.update(meta)
        context.update(kwargs)

        return context

    def render(self, context):
        raw_html = self.template.render(**context)
        base_url = os.path.dirname(self.template.filename)
        return HTML(string=raw_html, base_url=base_url)

    def get_filepath(self, **kwargs):
        filename = self.filename_format.format(**kwargs)
        return os.path.join(self.destination_path, filename)

    def generate_one(self, context):
        html = self.render(context)
        filepath = self.get_filepath(**context)
        html.write_pdf(filepath)

    def generate(self):
        data = self.get_certificate_data()
        for i, row in enumerate(data):
            context = self.get_context(id=i, **row)
            self.generate_one(context)


class CSVCertificator(BaseCertificator):
    def __init__(self, delimiter=',', meta_path='./meta.json', data_path='./data.csv', **kwargs):
        super().__init__(**kwargs)
        self.delimiter = delimiter
        self.meta_path = meta_path
        self.data_path = data_path

    def get_meta(self):
        with open(self.meta_path) as f:
            return json.loads(f.read())

    def get_certificate_data(self):
        with open(self.data_path) as f:
            return [row for row in csv.DictReader(f)]



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    install_requires = f.readlines()

setup(
    name='certificator',
    version='0.0.5',
    description='Certificate Generator Tool and Library',
    url='https://github.com/lamenezes/certificator',
    author='Luiz Menezes',
    author_email='luiz.menezesf@gmail.com',
    long_description=open('README.rst').read(),

    packages=find_packages(exclude=['tests*']),
    install_requires=install_requires,
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'certificator=certificator.__main__:generate',
        ]
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
)



import click

from .meetup import MeetupCertificator


@click.command()
@click.argument('destination', type=click.Path(exists=True, dir_okay=True), default='.')
@click.option('--urlname', '-u')
@click.option('--event_id', '-e')
@click.option('--meetup-api-key', '-k')
@click.option('--template-path', '-t', type=click.Path(exists=True, dir_okay=True), default=None)
def generate(destination, urlname, event_id, meetup_api_key, template_path):
    assert urlname and event_id and meetup_api_key, ('You must pass the --urlname, --event_id '
                                                     'and --meetup-api-key arguments')

    certifier = MeetupCertificator(
        urlname=urlname,
        event_id=event_id,
        destination_path=destination,
        template_path=template_path,
        api_key=meetup_api_key,
    )
    certifier.generate()


if __name__ == '__main__':
    generate()
